/* Petit programme pour tester le multiplexeur 16 canaux ADAFRUIT.

   Permet de déterminer les caractéristiques des moteurs. Pour couvrir à coup sûr la
   plage des 180° les impulsions étant volontairement un peu hors limites.
   Par défaut sur le multiplexeur l'adresse I2C sera 0x40. (Peut être modifiée sur le matériel.)

   UTILISER LE PROGRAMME : En faisant varier la tension sur A1 avec le potentiomètre on pilote
       tous les canaux simultanément. Ainsi on peut brancher un moteur sur n'importe quelle
       sortie pour vérifier son bon fonctionnement et surtout déterminer les valeurs
       limites à imposer au moteur pour le maintenir dans la plage mécanique et électrique normale.
       La valeur de consigne angulaire et pour Toff ainsi que la durée de l'impulsion "positive"
       sont affichées en permanence sur le Moniteur série de l'IDE.
       (Penser en ligne 44 à ajuster le débit série exprimé en Bauds.)
         
DIFFERENTS BRANCHEMENTS :
 A1 : Tention potentiométrique de commande. (0 à +5Vcc.)
 A4 -> SDA pour la ligne ligne I2C.
 A5 -> SCL pour la ligne ligne I2C.

                                                                PGM               RAM dynamique
 PGM avec affichage sur le Moniteur de l'IDE.              7168 Octets (22%)      519 Octets (25%)    */

//--------------------- Déclarations pour le matériel -----------------------
#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

//--------------------- Définition des broches d'E/S ------------------------
#define Entree_mesuree 1 // Entrée analogique A5 utilisée.

//------------------------ Constantes du programme --------------------------
#define NB_MAX  16 // Nombre se servomoteurs branchés.
#define NB_tests_antirebonds 50 // Nombre de tests de stabilisation B.P.

//------------------------- Variables du programme --------------------------
byte Num_moteur;
char Angle; // Char pour pouvoir faire varier entre -90 et +90;
int Pulse;
int CNA;
//-------------------------- Fin des déclarations ---------------------------

void setup() {
  Serial.begin(115200);
  Serial.println("Test sur les 16 caneaux et caracteristiques des moteurs.");
  pwm.begin(); // Initialise le module multiplexeur  
  pwm.setPWMFreq(50);}  // Fréquence de la PWM : En standard 50Hz pour des servomoteurs.

void loop() {
 // Commander les 16 sorties simultanément à la même valeur.
  CNA = analogRead(Entree_mesuree);
  Angle = map(CNA, 0, 1023, +90, -90); // Génère une valeur comprise entre -90 et +90.
  Serial.print("Angle = "); Serial.print(float(Angle));
  Pulse = map(Angle, -90, +90, 505, 100); // ATTENSION les limites sont hors standard.
  Num_moteur = 0;
  while (Num_moteur < NB_MAX)
      {pwm.setPWM(Num_moteur, 0, Pulse); Num_moteur ++;}
  Serial.print("  Consigne = "); Serial.print(Pulse);    
  Serial.print("  Impulsion = "); Serial.print(Pulse * 4.884,0); Serial.println(" micro S"); }
      

  

  
