/* Ce programme est conçu pour mettre au point le dialogue Homme/Machine ainsi que les
   protocoles. Emule l'analyseur syntaxique, génère les codes d'erreur, accuse réception,
   et aiguille les actions à conduire sur consigne cohérente.

   UTILISER LE PROGRAMME : Frapper une chaine terminée par "*" sur la ligne série puis
       observer l'accusé de réception et comparer au tableau des erreurs de syntaxe;
         
DIFFERENTS BRANCHEMENTS :
 A4 -> SDA pour la ligne ligne I2C.
 A5 -> SCL pour la ligne ligne I2C.
 D2  : Pilotage du BUZZER. (Optionnel le dialogue série accusant réception.

                                                                PGM               RAM dynamique
 PGM de dialogue sur la ligne série.                    7026 Octets (22%)      312 Octets (15%)    */

//--------------------- Définition des broches d'E/S ------------------------
#define BUZZER 2 // Le bruiteur est piloté par la sortie binaire D2.
//------------------------ Constantes du programme --------------------------
#define NB_MAX  18 // Nombre de servomoteurs branchés.
#define Sentinelle '*' // Choisir un caractère pour la "sentinelle".
#define LGR_chaine 7 // Longueur maximale de la chaine de dialogue.
#define PGMmax 3 // Nombre maximum de programmes.

//------------------------- Variables du programme --------------------------
boolean Sentinelle_detectee;  // Délimiteur de fin détecté.
boolean PGM; // Indique si la consigne invoque un Programme.
char Caractere;
byte Num_Sortie, Compteur_caracteres, num_ERR;
int NOMBRE; // Contiendra la transformée numérique d'une chaine.
int Angle; // Int pour pouvoir faire varier entre -90 et +90 et compatible string;
String Chaine_Memorisee; // Tampon mémoire pour la chaine entrante.

//-------------------------- Fin des déclarations ---------------------------

void setup() {
  Serial.begin(115200);
  Serial.print("FIN avec '");
  Serial.print(char(Sentinelle)); Serial.println("'");
  Chaine_Memorisee.reserve(LGR_chaine); }  // Reserve LGR_chaine octets pour la chaine.

void loop() {
  Attendre_une_chaine(); Serial.print(Chaine_Memorisee); Serial.print(" > ");
  Analyseur_syntaxique();
  if (num_ERR == 0) { Serial.println("!OK!");
                      if(PGM) Traite_Programme(); else Traite_moteur();}
  else ERREUR();}
  
void Traite_moteur() {
  Serial.println(); Serial.print("Consigne moteur "); Serial.print(Num_Sortie+1);
  Serial.print(" Position "); Serial.println(Angle);}

void Traite_Programme() {
  switch (NOMBRE) {
    case 1 : { Serial.println(); Serial.print("Traite PGM "); Serial.println(NOMBRE); break;}
    case 2 : { Serial.println(); Serial.print("Traite PGM "); Serial.println(NOMBRE); break;}
    case 3 : { Serial.println(); Serial.print("Traite PGM "); Serial.println(NOMBRE);}}}
    
void Analyseur_syntaxique() {
  num_ERR = 0; PGM = true; // Valeurs par défaut avant analyse.
  Compteur_caracteres = Chaine_Memorisee.length(); // Pointeur de caractère dans la chaine.
  if (Compteur_caracteres < 4) num_ERR = 1; Caractere = Chaine_Memorisee[0];  
  Compteur_caracteres--;
  if (Chaine_Memorisee[Compteur_caracteres]  != '*') num_ERR = 2;
  //========================== Analyse pour PROGRAMME ==========================
  // En sortie NOMBRE contiendra le nombre dans la chaine ou 00 si incorrect.
  if (Compteur_caracteres == 3) { // Chaine de 4 caractères donc consigne PROGRAMME.
     if (Caractere != 'p') num_ERR = 3;
     Decale_la_chaine(); Extraire_un_nombre();
     if ((NOMBRE == 0) || (NOMBRE > PGMmax)) num_ERR = 6; }
  //============================ Analyse pour MOTEUR ===========================
  else {
      PGM = false; if (Caractere != 'm') num_ERR = 4;
      Decale_la_chaine(); Extraire_un_nombre();
      if ((NOMBRE > NB_MAX) || (NOMBRE == 0)) num_ERR = 7;
      Num_Sortie = NOMBRE-1;
      Serial.print("Sortie "); Serial.print(Num_Sortie); Espace();
      Decale_la_chaine(); if (NOMBRE > 9) Decale_la_chaine();
      Caractere = Chaine_Memorisee[0];
      if(!((Caractere == '-') || (Caractere == '+'))) num_ERR = 5; 
      Extraire_un_nombre();
      Angle = NOMBRE; Serial.print("Angle "); Serial.print(Angle); Espace();
      if (Angle > 90) num_ERR = 8; if (Angle < -90) num_ERR = 9;} }
  
void Extraire_un_nombre() {// Extraction jusqu'à un caractère non chiffre.
  // L'extraction commence au début de Chaine_Memorisee.
  NOMBRE = Chaine_Memorisee.toInt();}
  
void Decale_la_chaine() {
  for (byte I = 0; I < LGR_chaine; I++) Chaine_Memorisee[I] = Chaine_Memorisee[I+1];}
  
void Espace() {Serial.print(" ");}
  
void ERREUR() {BIP(); Serial.print("!ERR "); Serial.print(num_ERR); Serial.println("!");}

void Attendre_une_chaine() {// Cette procédure attend une chaine sur la ligne USB.
  Chaine_Memorisee = ""; // Vider le tampon pour une nouvelle capture.
  Sentinelle_detectee = false;  // Sentinelle non reçue.
  Compteur_caracteres=0;
  while (!Sentinelle_detectee) {
        if (Serial.available()) {Caractere = Serial.read(); // Puiser un caractère dans le FIFO.
        // Puis concaténer ce caractère y compris la sentinelle :
        if (Compteur_caracteres < LGR_chaine) {Chaine_Memorisee += Caractere; Compteur_caracteres++;}
        // Si sentinelle détectée, informer la boucle de réception :
        Sentinelle_detectee = (Caractere == Sentinelle); }} }
  
void BIP() {
  pinMode(BUZZER, OUTPUT); tone(BUZZER,3000); delay(100); noTone(BUZZER); pinMode(BUZZER, INPUT);}
  
  
