/* Petit programme pour tester la mise en oeuvre du multiplexeur 16 canaux ADAFRUIT.

   Ce programme est conçu pour mettre au point la procédure de base qui sera chargée de
   provoquer la rotation d'un moteur individuellement tout en gérant ses butées logicielles.

   UTILISER LE PROGRAMME : Brancher les moteurs dans l'ordre c'est à dire le moteur n°1 sur la
   sortie S0, le moteur n°2 sur S1 etc jusqu'à moteur n°16 sur S15. Le tableau des bornes
   Butées[36] a été complété par les valeurs relatives à vos moteurs. Du reste si vous ne possédez
   que 12 moteurs il faudra le limiter à Butées[24] et mettre à jour NB_MAX.
   Mettre également à jour dans Programme_01() les valeurs des centres opérationnels.
   Avec le moniteur de l'IDE envoyez des consignes et vérifiez que les moteurs se positionnent
   correctement. Comme les assezvissements sont analogiques, une divergence d'environ 5°
   affectera les résultats pratiques même si le tableau de la Fig.26 a été rédigé avec
   grand soin. Testez également le programme "P01*".
   >>>> Pensez à terminer ces manipulation par l'appel à P01.
         
DIFFERENTS BRANCHEMENTS :
 A1 : Tention potentiométrique de commande. (0 à +5Vcc.)
 A4 -> SDA pour la ligne ligne I2C.
 A5 -> SCL pour la ligne ligne I2C.
 D2  : Pilotage du BUZZER. (Optionnel le dialogue série accusant réception.

                                                           PGM               RAM dynamique
 PGM avec dialogue série, routine de base et P01.   10698 Octets (34%)      595 Octets (29%)    */

//--------------------- Déclarations pour le matériel -----------------------
#include <Wire.h>
#include <Adafruit_PWMServoDriver.h>

Adafruit_PWMServoDriver pwm = Adafruit_PWMServoDriver();

//--------------------- Définition des broches d'E/S ------------------------
#define BUZZER 2 // Le bruiteur est piloté par la sortie binaire D2.

//------------------------ Constantes du programme --------------------------
#define NB_MAX  18 // Nombre de servomoteurs branchés.
#define Sentinelle '*' // Choisir un caractère pour la "sentinelle".
#define LGR_chaine 7 // Longueur maximale de la chaine de dialogue.
#define PGMmax 3 // Nombre maximum de programmes.

//------------------------- Variables du programme --------------------------
boolean Sentinelle_detectee;  // Délimiteur de fin détecté.
boolean PGM; // Indique si la consigne invoque un Programme.
char Caractere;
byte Num_Sortie, Compteur_caracteres, num_ERR;
int Angle; // Int pour pouvoir faire varier entre -90 et +90 et compatible string;
int NOMBRE; // Contiendra la transformée numérique d'une chaine.
int Pulse;
int CNA;
String Chaine_Memorisee; // Tampon mémoire pour la chaine entrante.
// ATTENTION : Dans le tableau les butées sont écrites dans l'hypothèse où les moteurs sont
//             branchés dans l'ordre croissant sur les sorties.  Le n°1 sur S0, le n° sur S1 etc.
//             Dans la sélection définitive et la répartition déterminée, il faudra impérativement
//             limiter ce tableau à 12 moteurs et préciser les valeurs idoines pour les butées.
int Butees[36] = {500,150,480,134,483,143,478,136,494,145,481,152,483,136,490,134,492,141,
                  481,154,485,139,472,139,460,137,503,154,487,145,470,132,487,159,474,143};
//-------------------------- Fin des déclarations ---------------------------

void setup() {
  Serial.begin(115200);
  Serial.print("FIN avec '"); Serial.print(char(Sentinelle)); Serial.println("'");
  Chaine_Memorisee.reserve(LGR_chaine); // Reserve LGR_chaine octets pour la chaine.
  pwm.begin(); // Initialise le module multiplexeur  
  pwm.setPWMFreq(50);}  // Fréquence de la PWM : En standard 50Hz pour des servomoteurs.

void loop() {
  Attendre_une_chaine(); Serial.println(); Serial.print(Chaine_Memorisee); Serial.print(" > ");
  Analyseur_syntaxique();
  if (num_ERR == 0) { Serial.println("!OK!");
                      if(PGM) Traite_Programme(); else Traite_moteur();}
  else ERREUR();}
  
void Traite_moteur() {
  byte Pointeur_butee; Pointeur_butee = Num_Sortie;
  Pointeur_butee = Pointeur_butee << 1; // Multiplie la valeur du pointeur par deux. 
  Pulse = map(Angle, -90, +90, Butees[Pointeur_butee], Butees[Pointeur_butee+1]);
  Serial.print("Moteur "); Serial.print(Num_Sortie+1);
  Serial.print(" Inf = "); Serial.print(Butees[Pointeur_butee]);
  Serial.print(" Sup = "); Serial.println(Butees[Pointeur_butee+1]);
  pwm.setPWM(Num_Sortie, 0, Pulse);}

void Traite_Programme() {
  switch (NOMBRE) {
    case 1 : { Programme_01(); break;}
    case 2 : { Serial.println(); Serial.print("Traite PGM "); Serial.println(NOMBRE); break;}
    case 3 : { Serial.println(); Serial.print("Traite PGM "); Serial.println(NOMBRE);}}}
    
void Programme_01() { // Cale tous les palonniers à la position 0°.
  int Neutre_operationnel;
  for (Num_Sortie = 0; Num_Sortie <12; Num_Sortie++) {
  switch (Num_Sortie) {
    case 0 : { Neutre_operationnel = 320; break;}
    case 1 : { Neutre_operationnel = 300; break;}
    case 2 : { Neutre_operationnel = 303; break;}
    case 3 : { Neutre_operationnel = 292; break;}
    case 4 : { Neutre_operationnel = 307; break;}
    case 5 : { Neutre_operationnel = 296; break;}
    case 6 : { Neutre_operationnel = 301; break;}
    case 7 : { Neutre_operationnel = 305; break;}
    case 8 : { Neutre_operationnel = 303; break;}
    case 9 : { Neutre_operationnel = 314; break;}
    case 10 : { Neutre_operationnel = 301; break;}
    case 11 : { Neutre_operationnel = 290;}} 
    pwm.setPWM(Num_Sortie, 0, Neutre_operationnel);}}
    
void Analyseur_syntaxique() {
  num_ERR = 0; PGM = true; // Valeurs par défaut avant analyse.
  Compteur_caracteres = Chaine_Memorisee.length(); // Pointeur de caractère dans la chaine.
  if (Compteur_caracteres < 4) num_ERR = 1; Caractere = Chaine_Memorisee[0];  
  Compteur_caracteres--;
  if (Chaine_Memorisee[Compteur_caracteres]  != '*') num_ERR = 2;
  //========================== Analyse pour PROGRAMME ==========================
  // En sortie NOMBRE contiendra le nombre dans la chaine ou 00 si incorrect.
  if (Compteur_caracteres == 3) { // Chaine de 4 caractères donc consigne PROGRAMME.
     if (Caractere != 'p') num_ERR = 3;
     Decale_la_chaine(); Extraire_un_nombre();
     if ((NOMBRE == 0) || (NOMBRE > PGMmax)) num_ERR = 6; }
  //============================ Analyse pour MOTEUR ===========================
  else {
      PGM = false; if (Caractere != 'm') num_ERR = 4;
      Decale_la_chaine(); Extraire_un_nombre();
      if ((NOMBRE > NB_MAX) || (NOMBRE == 0)) num_ERR = 7;
      Num_Sortie = NOMBRE-1;
      Serial.print("Sortie "); Serial.print(Num_Sortie); Espace();
      Decale_la_chaine(); if (NOMBRE > 9) Decale_la_chaine();
      Caractere = Chaine_Memorisee[0];
      if(!((Caractere == '-') || (Caractere == '+'))) num_ERR = 5;    
      Extraire_un_nombre();
      Angle = NOMBRE; Serial.print("Angle "); Serial.print(Angle); Espace();
      if (Angle > 90) num_ERR = 8; if (Angle < -90) num_ERR = 9;} }
  
void Extraire_un_nombre() {// Extraction jusqu'à un caractère non chiffre.
  // L'extraction commence au début de Chaine_Memorisee.
  NOMBRE = Chaine_Memorisee.toInt();}
  
void Decale_la_chaine() {
  for (byte I = 0; I < LGR_chaine; I++) Chaine_Memorisee[I] = Chaine_Memorisee[I+1];}
  
void Espace() {Serial.print(" ");}
  
void ERREUR() {BIP(); Serial.print("!ERR "); Serial.print(num_ERR); Serial.println("!");}

void Attendre_une_chaine() {// Cette procédure attend une chaine sur la ligne USB.
  Chaine_Memorisee = ""; // Vider le tampon pour une nouvelle capture.
  Sentinelle_detectee = false;  // Sentinelle non reçue.
  Compteur_caracteres=0;
  while (!Sentinelle_detectee) {
        if (Serial.available()) {Caractere = Serial.read(); // Puiser un caractère dans le FIFO.
        // Puis concaténer ce caractère y compris la sentinelle :
        if (Compteur_caracteres < LGR_chaine) {Chaine_Memorisee += Caractere; Compteur_caracteres++;}
        // Si sentinelle détectée, informer la boucle de réception :
        Sentinelle_detectee = (Caractere == Sentinelle); }} }
  
void BIP() {
  pinMode(BUZZER, OUTPUT); tone(BUZZER,3000); delay(100); noTone(BUZZER); pinMode(BUZZER, INPUT);}
  
  
      

  

  
